#!/bin/bash

###############################
function get_POST_vars()
{
	# save POST variables (only first time this is called)
	[ -z "$QUERY_STRING_POST" \
	  -a "$REQUEST_METHOD" = "POST" -a ! -z "$CONTENT_LENGTH" ] && \
		read -n $CONTENT_LENGTH QUERY_STRING_POST
	return
}


###############################
function decodevar()
{
    [ $# -ne 1 ] && return
    local v t h
    # replace all + with whitespace and append %%
    t="${1//+/ }%%"
    while [ ${#t} -gt 0 -a "${t}" != "%" ]; do
	v="${v}${t%%\%*}" # digest up to the first %
	t="${t#*%}"       # remove digested part
	# decode if there is anything to decode and if not at end of string
	if [ ${#t} -gt 0 -a "${t}" != "%" ]; then
	    h=${t:0:2} # save first two chars
	    t="${t:2}" # remove these
	    v="${v}"`echo -e \\\\x${h}` # convert hex to special char
	fi
    done
    # return decoded string
    echo "${v}"
    return
}


###############################
# routine to get variables from http requests
# usage: getvars
# returns: all http request variables converted into bash variables
function getvars()
{
	local q p k v
	# get query
	[ ! -z "${QUERY_STRING}" ] && q="${QUERY_STRING}&"
	get_POST_vars
	[ ! -z "${QUERY_STRING_POST}" ] && q="${q}${QUERY_STRING_POST}&"

	# parse the query data
	while [ ! -z "$q" ]; do
		p="${q%%&*}"  # get first part of query string
		k="${p%%=*}"  # get the key (variable name) from it
		v="${p#*=}"   # get the value from it
		q="${q#$p&*}" # strip first part from query string

		# decode and evaluate var
		eval "$k=\"`decodevar \"$v\"`\""
	done
	return
}


###############################
#
# Parse html request string
#
getvars

